defmodule DecentralisedBookIndex.Metadata.BookId do
  use Ash.Resource,
    otp_app: :decentralised_book_index,
    domain: DecentralisedBookIndex.Metadata,
    data_layer: AshPostgres.DataLayer

  alias DecentralisedBookIndex.Metadata

  postgres do
    table "book_ids"
    repo DecentralisedBookIndex.Repo
  end

  actions do
    defaults [:read, :update, :destroy]

    create :create do
      primary? true
      accept [:type, :bid, :order]
    end

    read :by_id do
      argument :id, :uuid, allow_nil?: false
      get? true
      filter expr(id == ^arg(:id))
    end
  end

  attributes do
    uuid_primary_key :id

    attribute :type, :string do
      allow_nil? false
      public? true
    end

    attribute :bid, :string do
      allow_nil? false
      public? true
    end

    attribute :order, :integer do
      allow_nil? false
      public? true
    end

    timestamps()
  end

  relationships do
    belongs_to :book, Metadata.Book do
      allow_nil? true
    end
  end
end
